-------------------------------------------------------------------------------
-- Product Note PN122
-- VHDL module used in the 'RTDS MMC SFP' project
-- Copyright imperix ltd, 2024
-- dev@imperix.ch
-------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------------------------------------
-- This driver contains:
-- * a trigger block to generate a trigger for the emitter at a given frequency (i.e. period in cycles received from the design)
-- * a firing words block to concatenate the PWM, fast discharge and deactivate signals into 8-bit firing words
-- * an emitter to cast the firing words into frames to be transmitted via the Aurora IP to the RTDS GTSOC V2
-- * a router to receive frames from the RTDS GTSOC V2 via the Aurora IP and redirect each frame to the right serial-to-parallel block
-- * a serial-to-parallel A block to decode the received A frames and update the corresponding registers
-- * a serial-to-parallel B block to decode the received B frames and update the corresponding registers
-- * a serial-to-parallel C block to decode the received C frames and update the corresponding registers
-----------------------------------------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------------------------------------
---- INPUTS
--
-- * clk : Main clock, assumed to run at 250 MHz.
--
-- * pwm : PWM signals, two per submodule (cf. RTDS GTSOC datasheet), to be sent to the RTDS GTSOC V2 through the Aurora 8B/10B IP.
--         Inserted into the 8-bit firing words, one per submodule, in the firing words block.
--         With 4 submodules per arm: pwm(0)  = pwm signal (0 or 1) for T1 in submodule 1 of arm 1
--                                    pwm(1)  = pwm signal (0 or 1) for T2 in submodule 1 of arm 1
--                                    pwm(2)  = pwm signal (0 or 1) for T1 in submodule 2 of arm 1
--                                    pwm(3)  = pwm signal (0 or 1) for T2 in submodule 2 of arm 1
--                                    (...)
--                                    pwm(46) = pwm signal (0 or 1) for T2 in submodule 4 of arm 6
--                                    pwm(47) = pwm signal (0 or 1) for T2 in submodule 4 of arm 6
--
-- * fast_discharge : Fast discharge signals (cf. RTDS GTSOC datasheet), one per submodule, to be sent to the RTDS GTSOC V2 through the Aurora 8B/10B IP.
--         Inserted into the 8-bit firing words, one per submodule, in the firing words block.
--         With 4 submodules per arm: fast_discharge(0)  = 1 if submodule 1 of arm 1 must be fast discharged, 0 otherwise
--                                    fast_discharge(1)  = 1 if submodule 2 of arm 1 must be fast discharged, 0 otherwise
--                                    (...)
--                                    fast_discharge(22) = 1 if submodule 3 of arm 6 must be fast discharged, 0 otherwise
--                                    fast_discharge(23) = 1 if submodule 4 of arm 6 must be fast discharged, 0 otherwise
--
-- * deactivate : Deactivate signals (cf. RTDS GTSOC datasheet), one per submodule, to be sent to the RTDS GTSOC V2 through the Aurora 8B/10B IP.
--         Inserted into the 8-bit firing words, one per submodule, in the firing words block.
--         With 4 submodules per arm: deactivate(0)  = 1 if submodule 1 of arm 1 must be deactivated, 0 otherwise
--                                    deactivate(1)  = 1 if submodule 2 of arm 1 must be deactivated, 0 otherwise
--                                    (...)
--                                    deactivate(22) = 1 if submodule 3 of arm 6 must be deactivated, 0 otherwise
--                                    deactivate(23) = 1 if submodule 4 of arm 6 must be deactivated, 0 otherwise
--
-- * sending_ticks : Number of FPGA cycles/ticks between two transmissions of the firing words to the RTDS GTSOC V2
--
-- * FROM_RTDS_S_AXIS : AXI-Stream interface to receive data from the RTDS GTSOC V2 through the Aurora 8B/10B block.
--         Must be connected to the USER_DATA_M_AXI_RX interface of the Aurora 8B/10B IP through a FIFO for the clock domain crossing.
--
---- OUTPUTS
--
-- * dbg_Nsm_rtds : Debug signal, indicates the number of submodules received from the RTDS GTSOC V2 in the header of packets B and C.
--
-- * dbg_Nsm_rtds_vld : Debug signal, valid signal of the dbg_Nsm_rtds, set to 1 for one cycle when the dbg_Nsm_rtds signal is updated.
--
-- * dbg_error_packet : Debug signal, set to 1 when the length of the received packet does not correspond to the expected length.
--         Must possibly be latched if used.
--
-- * dbg_error_tkeep : Debug signal, set to 1 when the tkeep signal of the FROM_RTDS_S_AXIS is not '1111' (not a full word).
--         Must possibly be latched if used.
--
-- * dbg_error_timing : Debug signal, set to 1 when a new emission is triggered although the previous emission is not finished.
--         Must possibly be latched if used.
--
-- * TO_RTDS_M_AXIS : AXI-Stream interface to send data to the RTDS GTSOC V2 through the Aurora 8B/10B block.
--         Must be connected to the USER_DATA_S_AXI_TX interface of the Aurora 8B/10B IP through a FIFO for the clock domain crossing.
--
-- * A_(...) : Quantities received from the RTDS GTSOC V2 in packet A.
--
-- * out_valid_A : Valid signal set to 1 for one cycle when the A_(...) outputs are updated.
--
-- * B_(...) : Quantities received from the RTDS GTSOC V2 in packet B.
--
-- * out_valid_B : Valid signal set to 1 for one cycle when the B_(...) outputs are updated.
--
-- * C_(...) : Quantities received from the RTDS GTSOC V2 in packet C.
--
-- * out_valid_C : Valid signal set to 1 for one cycle when the C_(...) outputs are updated.
-----------------------------------------------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RTDS_MMC is
    Generic(
        N : natural := 4
    );
    Port(
        clk : IN std_logic;

        -- user inputs
        pwm : IN std_logic_vector(2*N*6-1 downto 0);
        fast_discharge : IN std_logic_vector(N*6-1 downto 0);
        deactivate : IN std_logic_vector(N*6-1 downto 0);

        -- trigger period (in FPGA cycles/ticks)
        sending_ticks : IN unsigned(15 downto 0);

        -- debug outputs
        dbg_Nsm_rtds : OUT unsigned(11 downto 0);
        dbg_Nsm_rtds_vld : OUT std_logic;

        -- debug errors
        -- one-cycle pulses -> must be latched if used
        dbg_error_packet : OUT std_logic;
        dbg_error_tkeep : OUT std_logic;
        dbg_error_timing : OUT std_logic;

        -- from Aurora 8B/10B AXIS interface
        FROM_RTDS_S_AXIS_tdata : IN std_logic_vector(31 downto 0);
        FROM_RTDS_S_AXIS_tvalid : IN std_logic;
        FROM_RTDS_S_AXIS_tready : OUT std_logic;
        FROM_RTDS_S_AXIS_tlast : IN std_logic;
        FROM_RTDS_S_AXIS_tkeep : IN std_logic_vector(3 downto 0);

        -- to Aurora 8B/10B AXIS interface
        TO_RTDS_M_AXIS_tdata : OUT std_logic_vector(31 downto 0);
        TO_RTDS_M_AXIS_tvalid : OUT std_logic;
        TO_RTDS_M_AXIS_tready : IN std_logic;
        TO_RTDS_M_AXIS_tlast : OUT std_logic;
        TO_RTDS_M_AXIS_tkeep : OUT std_logic_vector(3 downto 0);

        -- data outputs (packet A)
        A_v_branch_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_i_branch_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_valves_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_branch_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_i_branch_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_valves_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_branch_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_i_branch_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_valves_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_branch_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_i_branch_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_valves_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_branch_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_i_branch_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_valves_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_branch_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_i_branch_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_v_valves_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');

        A_passthru_1_13  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_2_13  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_3_13  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_4_13  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_5_13  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_6_13  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_1_46  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_2_46  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_3_46  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_4_46  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_5_46  : OUT std_logic_vector(31 downto 0) := (others => '0');
        A_passthru_6_46  : OUT std_logic_vector(31 downto 0) := (others => '0');

        out_valid_A : OUT std_logic := '0';

        -- data outputs (packet B)
        B_cap_v_sm_01_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_02_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_03_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_04_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');

        B_cap_v_sm_01_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_02_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_03_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_04_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');

        B_cap_v_sm_01_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_02_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_03_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_04_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');

        B_cap_v_sm_01_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_02_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_03_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_04_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');

        B_cap_v_sm_01_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_02_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_03_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_04_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');

        B_cap_v_sm_01_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_02_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_03_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        B_cap_v_sm_04_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');

        out_valid_B : OUT std_logic := '0';

        -- data outputs (packet C)
        C_sm_02_01_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');
        C_sm_04_03_leg_1 : OUT std_logic_vector(31 downto 0) := (others => '0');

        C_sm_02_01_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');
        C_sm_04_03_leg_2 : OUT std_logic_vector(31 downto 0) := (others => '0');

        C_sm_02_01_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');
        C_sm_04_03_leg_3 : OUT std_logic_vector(31 downto 0) := (others => '0');

        C_sm_02_01_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');
        C_sm_04_03_leg_4 : OUT std_logic_vector(31 downto 0) := (others => '0');

        C_sm_02_01_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');
        C_sm_04_03_leg_5 : OUT std_logic_vector(31 downto 0) := (others => '0');

        C_sm_02_01_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');
        C_sm_04_03_leg_6 : OUT std_logic_vector(31 downto 0) := (others => '0');

        out_valid_C : OUT std_logic := '0'
    );
end RTDS_MMC;

architecture Behavioral of RTDS_MMC is

    constant nreset : std_logic := '1';

    signal trigger : std_logic;
    signal firing_words : std_logic_vector(N*6*8-1 downto 0);

    -- AXIS interface between router and serial_to_parallel (packet A)
    signal OUT_A_M_AXIS_tdata : std_logic_vector(31 downto 0) := (others => '0');
    signal OUT_A_M_AXIS_tvalid : std_logic := '0';
    signal OUT_A_M_AXIS_tlast : std_logic := '0';

    -- AXIS interface between router and serial_to_parallel (packet B)
    signal OUT_B_M_AXIS_tdata : std_logic_vector(31 downto 0) := (others => '0');
    signal OUT_B_M_AXIS_tvalid : std_logic := '0';
    signal OUT_B_M_AXIS_tlast : std_logic := '0';

    -- AXIS interface between router and serial_to_parallel (packet C)
    signal OUT_C_M_AXIS_tdata : std_logic_vector(31 downto 0) := (others => '0');
    signal OUT_C_M_AXIS_tvalid : std_logic := '0';
    signal OUT_C_M_AXIS_tlast : std_logic := '0';

    -- temp. signals because outputs cannot be used internally
    signal w_dbg_Nsm_rtds : unsigned(11 downto 0);

    -- interfaces
    ATTRIBUTE X_INTERFACE_INFO : STRING;
    ATTRIBUTE X_INTERFACE_PARAMETER : STRING;
    ATTRIBUTE X_INTERFACE_MODE : STRING;

    ATTRIBUTE X_INTERFACE_INFO of TO_RTDS_M_AXIS_tdata : SIGNAL is "xilinx.com:interface:axis:1.0 TO_RTDS_M_AXIS TDATA";
    ATTRIBUTE X_INTERFACE_INFO of TO_RTDS_M_AXIS_tvalid: SIGNAL is "xilinx.com:interface:axis:1.0 TO_RTDS_M_AXIS TVALID";
    ATTRIBUTE X_INTERFACE_INFO of TO_RTDS_M_AXIS_tready: SIGNAL is "xilinx.com:interface:axis:1.0 TO_RTDS_M_AXIS TREADY";
    ATTRIBUTE X_INTERFACE_INFO of TO_RTDS_M_AXIS_tlast : SIGNAL is "xilinx.com:interface:axis:1.0 TO_RTDS_M_AXIS TLAST";
    ATTRIBUTE X_INTERFACE_INFO of TO_RTDS_M_AXIS_tkeep : SIGNAL is "xilinx.com:interface:axis:1.0 TO_RTDS_M_AXIS TKEEP";

    ATTRIBUTE X_INTERFACE_INFO of FROM_RTDS_S_AXIS_tdata : SIGNAL is "xilinx.com:interface:axis:1.0 FROM_RTDS_S_AXIS TDATA";
    ATTRIBUTE X_INTERFACE_INFO of FROM_RTDS_S_AXIS_tvalid: SIGNAL is "xilinx.com:interface:axis:1.0 FROM_RTDS_S_AXIS TVALID";
    ATTRIBUTE X_INTERFACE_INFO of FROM_RTDS_S_AXIS_tready: SIGNAL is "xilinx.com:interface:axis:1.0 FROM_RTDS_S_AXIS TREADY";
    ATTRIBUTE X_INTERFACE_INFO of FROM_RTDS_S_AXIS_tlast : SIGNAL is "xilinx.com:interface:axis:1.0 FROM_RTDS_S_AXIS TLAST";
    ATTRIBUTE X_INTERFACE_INFO of FROM_RTDS_S_AXIS_tkeep : SIGNAL is "xilinx.com:interface:axis:1.0 FROM_RTDS_S_AXIS TKEEP";

    ATTRIBUTE X_INTERFACE_MODE of FROM_RTDS_S_AXIS_tdata : SIGNAL is "monitor";

    ATTRIBUTE X_INTERFACE_INFO of A_v_branch_leg_1   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_00";
    ATTRIBUTE X_INTERFACE_INFO of A_i_branch_leg_1   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_01";
    ATTRIBUTE X_INTERFACE_INFO of A_v_valves_leg_1   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_02";
    ATTRIBUTE X_INTERFACE_INFO of A_v_branch_leg_2   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_03";
    ATTRIBUTE X_INTERFACE_INFO of A_i_branch_leg_2   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_04";
    ATTRIBUTE X_INTERFACE_INFO of A_v_valves_leg_2   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_05";
    ATTRIBUTE X_INTERFACE_INFO of A_v_branch_leg_3   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_06";
    ATTRIBUTE X_INTERFACE_INFO of A_i_branch_leg_3   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_07";
    ATTRIBUTE X_INTERFACE_INFO of A_v_valves_leg_3   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_08";
    ATTRIBUTE X_INTERFACE_INFO of A_v_branch_leg_4   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_09";
    ATTRIBUTE X_INTERFACE_INFO of A_i_branch_leg_4   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_10";
    ATTRIBUTE X_INTERFACE_INFO of A_v_valves_leg_4   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_11";
    ATTRIBUTE X_INTERFACE_INFO of A_v_branch_leg_5   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_12";
    ATTRIBUTE X_INTERFACE_INFO of A_i_branch_leg_5   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_13";
    ATTRIBUTE X_INTERFACE_INFO of A_v_valves_leg_5   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_14";
    ATTRIBUTE X_INTERFACE_INFO of A_v_branch_leg_6   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_15";
    ATTRIBUTE X_INTERFACE_INFO of A_i_branch_leg_6   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_16";
    ATTRIBUTE X_INTERFACE_INFO of A_v_valves_leg_6   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_17";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_1_13    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_18";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_2_13    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_19";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_3_13    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_20";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_4_13    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_21";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_5_13    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_22";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_6_13    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_23";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_1_46    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_24";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_2_46    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_25";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_3_46    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_26";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_4_46    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_27";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_5_46    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_28";
    ATTRIBUTE X_INTERFACE_INFO of A_passthru_6_46    : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_29";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_01_leg_1: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_30";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_02_leg_1: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_31";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_03_leg_1: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_32";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_04_leg_1: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_33";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_01_leg_2: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_34";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_02_leg_2: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_35";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_03_leg_2: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_36";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_04_leg_2: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_37";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_01_leg_3: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_38";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_02_leg_3: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_39";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_03_leg_3: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_40";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_04_leg_3: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_41";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_01_leg_4: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_42";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_02_leg_4: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_43";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_03_leg_4: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_44";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_04_leg_4: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_45";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_01_leg_5: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_46";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_02_leg_5: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_47";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_03_leg_5: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_48";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_04_leg_5: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_49";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_01_leg_6: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_50";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_02_leg_6: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_51";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_03_leg_6: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_52";
    ATTRIBUTE X_INTERFACE_INFO of B_cap_v_sm_04_leg_6: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_53";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_02_01_leg_1   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_54";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_04_03_leg_1   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_55";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_02_01_leg_2   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_56";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_04_03_leg_2   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_57";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_02_01_leg_3   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_58";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_04_03_leg_3   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_59";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_02_01_leg_4   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_60";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_04_03_leg_4   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_61";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_02_01_leg_5   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_62";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_04_03_leg_5   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_63";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_02_01_leg_6   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_64";
    ATTRIBUTE X_INTERFACE_INFO of C_sm_04_03_leg_6   : SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 RTDS_data reg_65";

    ATTRIBUTE X_INTERFACE_INFO of dbg_Nsm_rtds:     SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 dbg reg_00";
    ATTRIBUTE X_INTERFACE_INFO of dbg_Nsm_rtds_vld: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 dbg reg_01";
    ATTRIBUTE X_INTERFACE_INFO of dbg_error_packet: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 dbg reg_02";
    ATTRIBUTE X_INTERFACE_INFO of dbg_error_tkeep:  SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 dbg reg_03";
    ATTRIBUTE X_INTERFACE_INFO of dbg_error_timing: SIGNAL is "imperix.ch:ix:user_regs_rtl:1.0 dbg reg_04";

    ATTRIBUTE X_INTERFACE_INFO of clk: SIGNAL is "xilinx.com:signal:clock:1.0 clk CLK";
    ATTRIBUTE X_INTERFACE_PARAMETER of clk: SIGNAL is "FREQ_HZ 250000000, ASSOCIATED_BUSIF TO_RTDS_M_AXIS:FROM_RTDS_S_AXIS:RTDS_data:dbg";

begin

    -----------------------------------------------------------------------------------------------------------------------------------------
    -- Legend: '--' = same interface as the previous signal
    -----------------------------------------------------------------------------------------------------------------------------------------

    rtds_trigger: entity work.RTDS_trigger port map(
        clk => clk,
        period => sending_ticks,                        -- from the design (input)
        trigger => trigger                              -- to the emitter
    );

    rtds_firing_words : entity work.RTDS_firing_words port map(
        clk => clk,
        pwm => pwm,                                     -- from the design (input)
        fast_discharge => fast_discharge,               -- from the design (input)
        deactivate => deactivate,                       -- from the design (input)
        firing_words => firing_words                    -- to the emitter
    );

    rtds_emitter : entity work.RTDS_emitter port map(
        clk => clk,
        nreset => nreset,
        trigger => trigger,                             -- from the trigger block
        firing_words => firing_words,                   -- from the firing words block
        OUT_M_AXIS_tdata => TO_RTDS_M_AXIS_tdata,       -- to the Aurora IP (output)
        OUT_M_AXIS_tvalid => TO_RTDS_M_AXIS_tvalid,     -- ..
        OUT_M_AXIS_tready => TO_RTDS_M_AXIS_tready,     -- ..
        OUT_M_AXIS_tlast => TO_RTDS_M_AXIS_tlast,       -- ..
        OUT_M_AXIS_tkeep => TO_RTDS_M_AXIS_tkeep,       -- ..
        error_timing => dbg_error_timing,               -- to the design (output)
        is_idle => open
    );

    rtds_router : entity work.RTDS_router port map(
        clk => clk,
        nreset => nreset,
        IN_S_AXIS_tdata => FROM_RTDS_S_AXIS_tdata,      -- from the Aurora IP (input)
        IN_S_AXIS_tvalid => FROM_RTDS_S_AXIS_tvalid,    -- ..
        IN_S_AXIS_tlast => FROM_RTDS_S_AXIS_tlast,      -- ..
        IN_S_AXIS_tready => FROM_RTDS_S_AXIS_tready,    -- ..
        IN_S_AXIS_tkeep => FROM_RTDS_S_AXIS_tkeep,      -- ..
        OUT_A_M_AXIS_tdata => OUT_A_M_AXIS_tdata,       -- to the serial-to-parallel A block
        OUT_A_M_AXIS_tvalid => OUT_A_M_AXIS_tvalid,     -- ..
        OUT_A_M_AXIS_tlast => OUT_A_M_AXIS_tlast,       -- ..
        OUT_B_M_AXIS_tdata => OUT_B_M_AXIS_tdata,       -- to the serial-to-parallel B block
        OUT_B_M_AXIS_tvalid => OUT_B_M_AXIS_tvalid,     -- ..
        OUT_B_M_AXIS_tlast => OUT_B_M_AXIS_tlast,       -- ..
        OUT_C_M_AXIS_tdata => OUT_C_M_AXIS_tdata,       -- to the serial-to-parallel C block
        OUT_C_M_AXIS_tvalid => OUT_C_M_AXIS_tvalid,     -- ..
        OUT_C_M_AXIS_tlast => OUT_C_M_AXIS_tlast,       -- ..
        Nsm_rtds => w_dbg_Nsm_rtds,                     -- to the design (output) (via dbg_Nsm_rtds)
        Nsm_rtds_vld => dbg_Nsm_rtds_vld,               -- to the design (output)
        error_packet => dbg_error_packet,               -- to the design (output)
        error_tkeep => dbg_error_tkeep                  -- to the design (output)
    );

    dbg_Nsm_rtds <= w_dbg_Nsm_rtds;                     -- to the design (output)

    rtds_serial_to_parallel_A : entity work.RTDS_serial_to_parallel_A port map(
        clk => clk,
        IN_S_AXIS_tdata => OUT_A_M_AXIS_tdata,          -- from the router
        IN_S_AXIS_tvalid => OUT_A_M_AXIS_tvalid,        -- ..
        IN_S_AXIS_tlast => OUT_A_M_AXIS_tlast,          -- ..
        v_branch_leg_1 => A_v_branch_leg_1,             -- to the design (output)
        i_branch_leg_1 => A_i_branch_leg_1,             -- to the design (output)
        v_valves_leg_1 => A_v_valves_leg_1,             -- to the design (output)
        v_branch_leg_2 => A_v_branch_leg_2,             -- to the design (output)
        i_branch_leg_2 => A_i_branch_leg_2,             -- to the design (output)
        v_valves_leg_2 => A_v_valves_leg_2,             -- to the design (output)
        v_branch_leg_3 => A_v_branch_leg_3,             -- to the design (output)
        i_branch_leg_3 => A_i_branch_leg_3,             -- to the design (output)
        v_valves_leg_3 => A_v_valves_leg_3,             -- to the design (output)
        v_branch_leg_4 => A_v_branch_leg_4,             -- to the design (output)
        i_branch_leg_4 => A_i_branch_leg_4,             -- to the design (output)
        v_valves_leg_4 => A_v_valves_leg_4,             -- to the design (output)
        v_branch_leg_5 => A_v_branch_leg_5,             -- to the design (output)
        i_branch_leg_5 => A_i_branch_leg_5,             -- to the design (output)
        v_valves_leg_5 => A_v_valves_leg_5,             -- to the design (output)
        v_branch_leg_6 => A_v_branch_leg_6,             -- to the design (output)
        i_branch_leg_6 => A_i_branch_leg_6,             -- to the design (output)
        v_valves_leg_6 => A_v_valves_leg_6,             -- to the design (output)
        passthru_1_13 => A_passthru_1_13,               -- to the design (output)
        passthru_2_13 => A_passthru_2_13,               -- to the design (output)
        passthru_3_13 => A_passthru_3_13,               -- to the design (output)
        passthru_4_13 => A_passthru_4_13,               -- to the design (output)
        passthru_5_13 => A_passthru_5_13,               -- to the design (output)
        passthru_6_13 => A_passthru_6_13,               -- to the design (output)
        passthru_1_46 => A_passthru_1_46,               -- to the design (output)
        passthru_2_46 => A_passthru_2_46,               -- to the design (output)
        passthru_3_46 => A_passthru_3_46,               -- to the design (output)
        passthru_4_46 => A_passthru_4_46,               -- to the design (output)
        passthru_5_46 => A_passthru_5_46,               -- to the design (output)
        passthru_6_46 => A_passthru_6_46,               -- to the design (output)
        out_valid => out_valid_A                        -- to the design (output)
    );

    rtds_serial_to_parallel_B : entity work.RTDS_serial_to_parallel_B port map(
        clk => clk,
        nreset => nreset,
        Nsm => w_dbg_Nsm_rtds(4 downto 0),              -- from the router
        IN_S_AXIS_tdata => OUT_B_M_AXIS_tdata,          -- from the router
        IN_S_AXIS_tvalid => OUT_B_M_AXIS_tvalid,        -- ..
        IN_S_AXIS_tlast => OUT_B_M_AXIS_tlast,          -- ..
        cap_v_sm_01_leg_1 => B_cap_v_sm_01_leg_1,       -- to the design (output)
        cap_v_sm_02_leg_1 => B_cap_v_sm_02_leg_1,       -- to the design (output)
        cap_v_sm_03_leg_1 => B_cap_v_sm_03_leg_1,       -- to the design (output)
        cap_v_sm_04_leg_1 => B_cap_v_sm_04_leg_1,       -- to the design (output)
        cap_v_sm_05_leg_1 => open,
        cap_v_sm_06_leg_1 => open,
        cap_v_sm_07_leg_1 => open,
        cap_v_sm_08_leg_1 => open,
        cap_v_sm_09_leg_1 => open,
        cap_v_sm_10_leg_1 => open,
        cap_v_sm_11_leg_1 => open,
        cap_v_sm_12_leg_1 => open,
        cap_v_sm_13_leg_1 => open,
        cap_v_sm_14_leg_1 => open,
        cap_v_sm_15_leg_1 => open,
        cap_v_sm_16_leg_1 => open,
        cap_v_sm_17_leg_1 => open,
        cap_v_sm_18_leg_1 => open,
        cap_v_sm_19_leg_1 => open,
        cap_v_sm_20_leg_1 => open,
        cap_v_sm_01_leg_2 => B_cap_v_sm_01_leg_2,       -- to the design (output)
        cap_v_sm_02_leg_2 => B_cap_v_sm_02_leg_2,       -- to the design (output)
        cap_v_sm_03_leg_2 => B_cap_v_sm_03_leg_2,       -- to the design (output)
        cap_v_sm_04_leg_2 => B_cap_v_sm_04_leg_2,       -- to the design (output)
        cap_v_sm_05_leg_2 => open,
        cap_v_sm_06_leg_2 => open,
        cap_v_sm_07_leg_2 => open,
        cap_v_sm_08_leg_2 => open,
        cap_v_sm_09_leg_2 => open,
        cap_v_sm_10_leg_2 => open,
        cap_v_sm_11_leg_2 => open,
        cap_v_sm_12_leg_2 => open,
        cap_v_sm_13_leg_2 => open,
        cap_v_sm_14_leg_2 => open,
        cap_v_sm_15_leg_2 => open,
        cap_v_sm_16_leg_2 => open,
        cap_v_sm_17_leg_2 => open,
        cap_v_sm_18_leg_2 => open,
        cap_v_sm_19_leg_2 => open,
        cap_v_sm_20_leg_2 => open,
        cap_v_sm_01_leg_3 => B_cap_v_sm_01_leg_3,       -- to the design (output)
        cap_v_sm_02_leg_3 => B_cap_v_sm_02_leg_3,       -- to the design (output)
        cap_v_sm_03_leg_3 => B_cap_v_sm_03_leg_3,       -- to the design (output)
        cap_v_sm_04_leg_3 => B_cap_v_sm_04_leg_3,       -- to the design (output)
        cap_v_sm_05_leg_3 => open,
        cap_v_sm_06_leg_3 => open,
        cap_v_sm_07_leg_3 => open,
        cap_v_sm_08_leg_3 => open,
        cap_v_sm_09_leg_3 => open,
        cap_v_sm_10_leg_3 => open,
        cap_v_sm_11_leg_3 => open,
        cap_v_sm_12_leg_3 => open,
        cap_v_sm_13_leg_3 => open,
        cap_v_sm_14_leg_3 => open,
        cap_v_sm_15_leg_3 => open,
        cap_v_sm_16_leg_3 => open,
        cap_v_sm_17_leg_3 => open,
        cap_v_sm_18_leg_3 => open,
        cap_v_sm_19_leg_3 => open,
        cap_v_sm_20_leg_3 => open,
        cap_v_sm_01_leg_4 => B_cap_v_sm_01_leg_4,       -- to the design (output)
        cap_v_sm_02_leg_4 => B_cap_v_sm_02_leg_4,       -- to the design (output)
        cap_v_sm_03_leg_4 => B_cap_v_sm_03_leg_4,       -- to the design (output)
        cap_v_sm_04_leg_4 => B_cap_v_sm_04_leg_4,       -- to the design (output)
        cap_v_sm_05_leg_4 => open,
        cap_v_sm_06_leg_4 => open,
        cap_v_sm_07_leg_4 => open,
        cap_v_sm_08_leg_4 => open,
        cap_v_sm_09_leg_4 => open,
        cap_v_sm_10_leg_4 => open,
        cap_v_sm_11_leg_4 => open,
        cap_v_sm_12_leg_4 => open,
        cap_v_sm_13_leg_4 => open,
        cap_v_sm_14_leg_4 => open,
        cap_v_sm_15_leg_4 => open,
        cap_v_sm_16_leg_4 => open,
        cap_v_sm_17_leg_4 => open,
        cap_v_sm_18_leg_4 => open,
        cap_v_sm_19_leg_4 => open,
        cap_v_sm_20_leg_4 => open,
        cap_v_sm_01_leg_5 => B_cap_v_sm_01_leg_5,       -- to the design (output)
        cap_v_sm_02_leg_5 => B_cap_v_sm_02_leg_5,       -- to the design (output)
        cap_v_sm_03_leg_5 => B_cap_v_sm_03_leg_5,       -- to the design (output)
        cap_v_sm_04_leg_5 => B_cap_v_sm_04_leg_5,       -- to the design (output)
        cap_v_sm_05_leg_5 => open,
        cap_v_sm_06_leg_5 => open,
        cap_v_sm_07_leg_5 => open,
        cap_v_sm_08_leg_5 => open,
        cap_v_sm_09_leg_5 => open,
        cap_v_sm_10_leg_5 => open,
        cap_v_sm_11_leg_5 => open,
        cap_v_sm_12_leg_5 => open,
        cap_v_sm_13_leg_5 => open,
        cap_v_sm_14_leg_5 => open,
        cap_v_sm_15_leg_5 => open,
        cap_v_sm_16_leg_5 => open,
        cap_v_sm_17_leg_5 => open,
        cap_v_sm_18_leg_5 => open,
        cap_v_sm_19_leg_5 => open,
        cap_v_sm_20_leg_5 => open,
        cap_v_sm_01_leg_6 => B_cap_v_sm_01_leg_6,       -- to the design (output)
        cap_v_sm_02_leg_6 => B_cap_v_sm_02_leg_6,       -- to the design (output)
        cap_v_sm_03_leg_6 => B_cap_v_sm_03_leg_6,       -- to the design (output)
        cap_v_sm_04_leg_6 => B_cap_v_sm_04_leg_6,       -- to the design (output)
        cap_v_sm_05_leg_6 => open,
        cap_v_sm_06_leg_6 => open,
        cap_v_sm_07_leg_6 => open,
        cap_v_sm_08_leg_6 => open,
        cap_v_sm_09_leg_6 => open,
        cap_v_sm_10_leg_6 => open,
        cap_v_sm_11_leg_6 => open,
        cap_v_sm_12_leg_6 => open,
        cap_v_sm_13_leg_6 => open,
        cap_v_sm_14_leg_6 => open,
        cap_v_sm_15_leg_6 => open,
        cap_v_sm_16_leg_6 => open,
        cap_v_sm_17_leg_6 => open,
        cap_v_sm_18_leg_6 => open,
        cap_v_sm_19_leg_6 => open,
        cap_v_sm_20_leg_6 => open,
        out_valid => out_valid_B                        -- to the design (output)
    );

    rtds_serial_to_parallel_C : entity work.RTDS_serial_to_parallel_C port map(
        clk => clk,
        nreset => nreset,
        Nsm => w_dbg_Nsm_rtds(4 downto 0),              -- from the router
        IN_S_AXIS_tdata => OUT_C_M_AXIS_tdata,          -- from the router
        IN_S_AXIS_tvalid => OUT_C_M_AXIS_tvalid,        -- ..
        IN_S_AXIS_tlast => OUT_C_M_AXIS_tlast,          -- ..
        sm_02_01_leg_1 => C_sm_02_01_leg_1,             -- to the design (output)
        sm_04_03_leg_1 => C_sm_04_03_leg_1,             -- to the design (output)
        sm_06_05_leg_1 => open,
        sm_08_07_leg_1 => open,
        sm_10_09_leg_1 => open,
        sm_12_11_leg_1 => open,
        sm_14_13_leg_1 => open,
        sm_16_15_leg_1 => open,
        sm_18_17_leg_1 => open,
        sm_20_19_leg_1 => open,
        sm_02_01_leg_2 => C_sm_02_01_leg_2,             -- to the design (output)
        sm_04_03_leg_2 => C_sm_04_03_leg_2,             -- to the design (output)
        sm_06_05_leg_2 => open,
        sm_08_07_leg_2 => open,
        sm_10_09_leg_2 => open,
        sm_12_11_leg_2 => open,
        sm_14_13_leg_2 => open,
        sm_16_15_leg_2 => open,
        sm_18_17_leg_2 => open,
        sm_20_19_leg_2 => open,
        sm_02_01_leg_3 => C_sm_02_01_leg_3,             -- to the design (output)
        sm_04_03_leg_3 => C_sm_04_03_leg_3,             -- to the design (output)
        sm_06_05_leg_3 => open,
        sm_08_07_leg_3 => open,
        sm_10_09_leg_3 => open,
        sm_12_11_leg_3 => open,
        sm_14_13_leg_3 => open,
        sm_16_15_leg_3 => open,
        sm_18_17_leg_3 => open,
        sm_20_19_leg_3 => open,
        sm_02_01_leg_4 => C_sm_02_01_leg_4,             -- to the design (output)
        sm_04_03_leg_4 => C_sm_04_03_leg_4,             -- to the design (output)
        sm_06_05_leg_4 => open,
        sm_08_07_leg_4 => open,
        sm_10_09_leg_4 => open,
        sm_12_11_leg_4 => open,
        sm_14_13_leg_4 => open,
        sm_16_15_leg_4 => open,
        sm_18_17_leg_4 => open,
        sm_20_19_leg_4 => open,
        sm_02_01_leg_5 => C_sm_02_01_leg_5,             -- to the design (output)
        sm_04_03_leg_5 => C_sm_04_03_leg_5,             -- to the design (output)
        sm_06_05_leg_5 => open,
        sm_08_07_leg_5 => open,
        sm_10_09_leg_5 => open,
        sm_12_11_leg_5 => open,
        sm_14_13_leg_5 => open,
        sm_16_15_leg_5 => open,
        sm_18_17_leg_5 => open,
        sm_20_19_leg_5 => open,
        sm_02_01_leg_6 => C_sm_02_01_leg_6,             -- to the design (output)
        sm_04_03_leg_6 => C_sm_04_03_leg_6,             -- to the design (output)
        sm_06_05_leg_6 => open,
        sm_08_07_leg_6 => open,
        sm_10_09_leg_6 => open,
        sm_12_11_leg_6 => open,
        sm_14_13_leg_6 => open,
        sm_16_15_leg_6 => open,
        sm_18_17_leg_6 => open,
        sm_20_19_leg_6 => open,
        out_valid => out_valid_C                        -- to the design (output)
    );

end Behavioral;
