-------------------------------------------------------------------------------
-- Product Note PN122
-- VHDL module used in the 'RTDS MMC SFP' project
-- Copyright imperix ltd, 2024
-- dev@imperix.ch
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RTDS_emitter is
    Generic(
        N : natural := 4
    );
    Port(
        clk : IN std_logic;
        nreset : IN std_logic;
        trigger : IN std_logic;

        -- 8-bit firing words, one per submodule,
        -- to be sent to the RTDS GTSOC V2
        firing_words : IN std_logic_vector(8*N*6-1 downto 0);

        OUT_M_AXIS_tdata : OUT std_logic_vector(31 downto 0);
        OUT_M_AXIS_tvalid : OUT std_logic;
        OUT_M_AXIS_tready : IN std_logic;
        OUT_M_AXIS_tlast : OUT std_logic;
        OUT_M_AXIS_tkeep : OUT std_logic_vector(3 downto 0) := "1111";

        -- error when emission is triggered although
        -- the previous emission is not finished
        error_timing : OUT std_logic := '0';

        is_idle : OUT std_logic := '0'
    );
end RTDS_emitter;

architecture Behavioral of RTDS_emitter is

    -- used to change the endianness
    signal w_OUT_M_AXIS_tdata : std_logic_vector(31 downto 0);

    -- n_words = number of 32-bit words to be sent to the RTDS
    -- as firing words are sent 4 by 4, ceil(N/4)=(N+3)>>2 32-bit words are required
    signal n_words : unsigned(15 downto 0) := shift_right((to_unsigned(N, 16)+3), 2);

    -- 12 is an arbitrary limit, could be increased if more
    -- than 4x12=48 submodules per arm are instantiated
    -- 32 is due to the 32-bit words that are transmitted
    subtype words_t is std_logic_vector(12*32-1 downto 0);
    signal words_leg1 : words_t := (others => '0');
    signal words_leg2 : words_t := (others => '0');
    signal words_leg3 : words_t := (others => '0');
    signal words_leg4 : words_t := (others => '0');
    signal words_leg5 : words_t := (others => '0');
    signal words_leg6 : words_t := (others => '0');

    type FSM_state_t is (IDLE, LEG_1, LEG_2, LEG_3, LEG_4, LEG_5, LEG_6);
    signal fsm_state : FSM_state_t := IDLE;
    signal fsm_next_state : FSM_state_t;

    signal current_header : std_logic_vector(31 downto 0);
    signal current_words : words_t;

    signal current_index : integer := 0;
    signal current_word : std_logic_vector(31 downto 0);
    
    signal leg_finished : std_logic := '0';

    signal w_error : std_logic := '0';

begin

    process(clk) begin
        if rising_edge(clk) then
            if trigger = '1' then
                words_leg1(N*8-1 downto 0) <= firing_words(1*N*8-1 downto 0*N*8);
                words_leg2(N*8-1 downto 0) <= firing_words(2*N*8-1 downto 1*N*8);
                words_leg3(N*8-1 downto 0) <= firing_words(3*N*8-1 downto 2*N*8);
                words_leg4(N*8-1 downto 0) <= firing_words(4*N*8-1 downto 3*N*8);
                words_leg5(N*8-1 downto 0) <= firing_words(5*N*8-1 downto 4*N*8);
                words_leg6(N*8-1 downto 0) <= firing_words(6*N*8-1 downto 5*N*8);
            end if;
        end if;
    end process;

    with fsm_state select fsm_next_state <=
        LEG_1 when IDLE, LEG_2 when LEG_1, LEG_3 when LEG_2,
        LEG_4 when LEG_3, LEG_5 when LEG_4, LEG_6 when LEG_5,
        IDLE when LEG_6, IDLE when others;

    with fsm_state select current_header <=
        x"10000000" when LEG_1, x"20000000" when LEG_2,
        x"30000000" when LEG_3, x"40000000" when LEG_4,
        x"50000000" when LEG_5, x"60000000" when LEG_6,
        x"00000000" when others;

    with fsm_state select current_words <=
        words_leg1 when LEG_1, words_leg2 when LEG_2,
        words_leg3 when LEG_3, words_leg4 when LEG_4,
        words_leg5 when LEG_5, words_leg6 when LEG_6,
        (others => '0') when others;
    
    -- error when emission is triggered although
    -- the previous emission is not finished
    w_error <= '1' when (trigger = '1') and (fsm_state /= IDLE) else '0';
    error_timing <= w_error;

    leg_finished <= '1' when (current_index = to_integer(n_words)) and (OUT_M_AXIS_tready = '1') else '0';

    process(clk) begin
        if rising_edge(clk) then

            -- FSM state update
            if nreset = '0' then
                fsm_state <= IDLE;
            elsif trigger = '1' then
                fsm_state <= LEG_1;
            else
                case fsm_state is
                    when LEG_1 | LEG_2 | LEG_3 | LEG_4 | LEG_5 | LEG_6 =>
                        if leg_finished = '1' then
                            fsm_state <= fsm_next_state;
                        end if;
                    when others =>
                end case;
            end if;

            -- index update
            -- index = 0 if reset, error, idle or leg finished
            -- index = index+1 otherwise
            if (nreset = '0') or (trigger = '1') then
                current_index <= 0;
            else
                case fsm_state is
                    when IDLE =>
                        current_index <= 0;
                    when others =>
                        if OUT_M_AXIS_tready = '1' then
                            if current_index = to_integer(n_words) then
                                current_index <= 0;
                            else
                                current_index <= current_index+1;
                            end if;
                        end if;
                end case;
            end if;

        end if;
    end process;
    
    -- get current word
    process(nreset, fsm_state, current_index, current_header, current_words) begin
        if (nreset = '0') or (fsm_state = IDLE) then
            current_word <= (others => '0');
        elsif current_index = 0 then
            current_word <= current_header;
        else
            current_word <= current_words(current_index*32-1 downto (current_index-1)*32);
        end if;
    end process;

    is_idle <= '1' when fsm_state = IDLE else '0';

    -- AXIS interface

    -- waiting for ready already taken into account in the FSM mechanism
    -- because counter (and, thus, FSM state) untouched when ready=0

    -- emitting full 32-bit words only
    OUT_M_AXIS_tkeep <= "1111";

    -- data is always current_word
    w_OUT_M_AXIS_tdata <= current_word;

    -- change endianness
    OUT_M_AXIS_tdata(31 downto 24) <= w_OUT_M_AXIS_tdata(7 downto 0);
    OUT_M_AXIS_tdata(23 downto 16) <= w_OUT_M_AXIS_tdata(15 downto 8);
    OUT_M_AXIS_tdata(15 downto  8) <= w_OUT_M_AXIS_tdata(23 downto 16);
    OUT_M_AXIS_tdata(7  downto  0) <= w_OUT_M_AXIS_tdata(31 downto 24);

    -- data is valid if state=LEG_i (-> not IDLE)
    OUT_M_AXIS_tvalid <= '1' when (fsm_state /= IDLE) else '0';

    -- data is last if state=LEG_6 and index=n_words
    OUT_M_AXIS_tlast <= '1' when current_index = to_integer(n_words) else '0';

end Behavioral;
