-------------------------------------------------------------------------------
-- Product Note PN122
-- VHDL module used in the 'RTDS MMC SFP' project
-- Copyright imperix ltd, 2024
-- dev@imperix.ch
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RTDS_router is
    Port(
        clk : IN std_logic;
        nreset : IN std_logic;

        -- input AXIS interface from the Aurora IP
        IN_S_AXIS_tdata : IN std_logic_vector(31 downto 0);
        IN_S_AXIS_tvalid : IN std_logic;
        IN_S_AXIS_tlast : IN std_logic;
        IN_S_AXIS_tready : OUT std_logic;
        IN_S_AXIS_tkeep : IN std_logic_vector(3 downto 0);

        -- output AXIS interface (packet A)
        OUT_A_M_AXIS_tdata : OUT std_logic_vector(31 downto 0) := (others => '0');
        OUT_A_M_AXIS_tvalid : OUT std_logic := '0';
        OUT_A_M_AXIS_tlast : OUT std_logic := '0';

        -- output AXIS interface (packet B)
        OUT_B_M_AXIS_tdata : OUT std_logic_vector(31 downto 0) := (others => '0');
        OUT_B_M_AXIS_tvalid : OUT std_logic := '0';
        OUT_B_M_AXIS_tlast : OUT std_logic := '0';

        -- output AXIS interface (packet C)
        OUT_C_M_AXIS_tdata : OUT std_logic_vector(31 downto 0) := (others => '0');
        OUT_C_M_AXIS_tvalid : OUT std_logic := '0';
        OUT_C_M_AXIS_tlast : OUT std_logic := '0';

        -- Nsm received from the RTDS
        Nsm_rtds : OUT unsigned(11 downto 0);
        Nsm_rtds_vld : OUT std_logic;

        -- error if number of received words in current packet
        -- does not correspond to the expected number of words
        error_packet : OUT std_logic := '0';

        -- error if keep is not '1111' because only
        -- expecting full 32b words only
        error_tkeep : OUT std_logic := '0'
    );
end RTDS_router;

architecture Behavioral of RTDS_router is

    -- used to change the endianness
    signal w_IN_S_AXIS_tdata : std_logic_vector(31 downto 0);

    -- Nsm received from the RTDS
    signal w_nsm_rtds : unsigned(11 downto 0) := x"000";
    signal w_nsm_rtds_vld : std_logic := '0';

    -- expected headers
    -- 3 first bytes of header B and C are containing Nsm_rtds
    constant header_A : std_logic_vector(31 downto 0) := x"00000000";
    constant header_B : std_logic_vector(19 downto 0) := x"10001";
    constant header_C : std_logic_vector(19 downto 0) := x"20001";

    -- expected packet lengths
    constant len_0 : unsigned(31 downto 0) := x"00000000";
    constant len_A : unsigned(31 downto 0) := x"0000001E";
    signal len_B : unsigned(31 downto 0);
    signal len_C : unsigned(31 downto 0);

    type FSM_state is (IDLE, WAIT_ANY, RCV_A, RCV_B, RCV_C);
    signal current_state : FSM_state := IDLE;

    signal transmission_valid : std_logic := '0';
    signal transmission_valid_last : std_logic := '0';

    signal counter : unsigned(31 downto 0) := x"00000000";
    signal current_len : unsigned(31 downto 0) := x"00000000";

    signal w_IN_S_AXIS_tready : std_logic;

begin

    -- change endianness
    w_IN_S_AXIS_tdata(31 downto 24) <= IN_S_AXIS_tdata(7 downto 0);
    w_IN_S_AXIS_tdata(23 downto 16) <= IN_S_AXIS_tdata(15 downto 8);
    w_IN_S_AXIS_tdata(15 downto  8) <= IN_S_AXIS_tdata(23 downto 16);
    w_IN_S_AXIS_tdata(7  downto  0) <= IN_S_AXIS_tdata(31 downto 24);

    -- length of packet B
    -- Nsm=20 is the maximal number of SMs considered in this model
    with w_nsm_rtds select len_B <=
        to_unsigned(6  , 32) when to_unsigned(1 , 12),
        to_unsigned(12 , 32) when to_unsigned(2 , 12),
        to_unsigned(18 , 32) when to_unsigned(3 , 12),
        to_unsigned(24 , 32) when to_unsigned(4 , 12),
        to_unsigned(30 , 32) when to_unsigned(5 , 12),
        to_unsigned(36 , 32) when to_unsigned(6 , 12),
        to_unsigned(42 , 32) when to_unsigned(7 , 12),
        to_unsigned(48 , 32) when to_unsigned(8 , 12),
        to_unsigned(54 , 32) when to_unsigned(9 , 12),
        to_unsigned(60 , 32) when to_unsigned(10, 12),
        to_unsigned(66 , 32) when to_unsigned(11, 12),
        to_unsigned(72 , 32) when to_unsigned(12, 12),
        to_unsigned(78 , 32) when to_unsigned(13, 12),
        to_unsigned(84 , 32) when to_unsigned(14, 12),
        to_unsigned(90 , 32) when to_unsigned(15, 12),
        to_unsigned(96 , 32) when to_unsigned(16, 12),
        to_unsigned(102, 32) when to_unsigned(17, 12),
        to_unsigned(108, 32) when to_unsigned(18, 12),
        to_unsigned(114, 32) when to_unsigned(19, 12),
        to_unsigned(120, 32) when to_unsigned(20, 12),
        to_unsigned(120, 32) when others;

    -- length of packet B
    -- Nsm=20 is the maximal number of SMs considered in this model
    with w_nsm_rtds select len_C <=
        to_unsigned(6 , 32) when to_unsigned(1 , 12),
        to_unsigned(6 , 32) when to_unsigned(2 , 12),
        to_unsigned(12, 32) when to_unsigned(3 , 12),
        to_unsigned(12, 32) when to_unsigned(4 , 12),
        to_unsigned(18, 32) when to_unsigned(5 , 12),
        to_unsigned(18, 32) when to_unsigned(6 , 12),
        to_unsigned(24, 32) when to_unsigned(7 , 12),
        to_unsigned(24, 32) when to_unsigned(8 , 12),
        to_unsigned(30, 32) when to_unsigned(9 , 12),
        to_unsigned(30, 32) when to_unsigned(10, 12),
        to_unsigned(36, 32) when to_unsigned(11, 12),
        to_unsigned(36, 32) when to_unsigned(12, 12),
        to_unsigned(42, 32) when to_unsigned(13, 12),
        to_unsigned(42, 32) when to_unsigned(14, 12),
        to_unsigned(48, 32) when to_unsigned(15, 12),
        to_unsigned(48, 32) when to_unsigned(16, 12),
        to_unsigned(54, 32) when to_unsigned(17, 12),
        to_unsigned(54, 32) when to_unsigned(18, 12),
        to_unsigned(60, 32) when to_unsigned(19, 12),
        to_unsigned(60, 32) when to_unsigned(20, 12),
        to_unsigned(60, 32) when others;

    -- always accepts data
    w_IN_S_AXIS_tready <= '1';
    IN_S_AXIS_tready <= w_IN_S_AXIS_tready;

    -- transmission is valid if valid and ready
    -- transmission is last if valid, ready and last
    transmission_valid <= '1' when (IN_S_AXIS_tvalid = '1') and (w_IN_S_AXIS_tready = '1') else '0';
    transmission_valid_last <= '1' when (transmission_valid = '1') and (IN_S_AXIS_tlast = '1') else '0';

    -- error if tkeep != '1111' when valid = 1
    -- expecting full 32-bit words only
    error_tkeep <= '1' when (IN_S_AXIS_tvalid = '1') and (IN_S_AXIS_tkeep /= "1111") else '0';

    -- FSM state update
    process(clk) begin
        if rising_edge(clk) then

            w_nsm_rtds_vld <= '0';

            case current_state is
                when IDLE | WAIT_ANY =>
                    if nreset = '0' then
                        current_state <= IDLE;
                    elsif transmission_valid = '1' then
                        if std_match(w_IN_S_AXIS_tdata, header_A) then
                            current_state <= RCV_A;
                        elsif std_match(w_IN_S_AXIS_tdata(19 downto 0), header_B) then
                            current_state <= RCV_B;
                            w_nsm_rtds <= unsigned(w_IN_S_AXIS_tdata(31 downto 20));
                            w_nsm_rtds_vld <= '1';
                        elsif std_match(w_IN_S_AXIS_tdata(19 downto 0), header_C) then
                            current_state <= RCV_C;
                            w_nsm_rtds <= unsigned(w_IN_S_AXIS_tdata(31 downto 20));
                            w_nsm_rtds_vld <= '1';
                        else
                            current_state <= current_state;
                        end if;
                    end if;

                when RCV_A =>
                    if nreset = '0' then
                        current_state <= IDLE;
                    elsif (transmission_valid_last = '1') then
                        current_state <= WAIT_ANY;
                    end if;

                when RCV_B =>
                    if nreset = '0' then
                        current_state <= IDLE;
                    elsif (transmission_valid_last = '1') then
                        current_state <= WAIT_ANY;
                    end if;

                when RCV_C =>
                    if nreset = '0' then
                        current_state <= IDLE;
                    elsif (transmission_valid_last = '1') then
                        current_state <= WAIT_ANY;
                    end if;
                
                when others =>
                    current_state <= current_state;
                    
            end case;
        end if;
    end process;

    -- expected packet length
    with current_state select current_len <=
        len_A when RCV_A,
        len_B when RCV_B,
        len_C when RCV_C,
        len_0 when others;

    -- count the number of received words in current packet
    -- and raise an error if this number doesn't correspond
    -- to the expected number of words in current packet
    process(clk) begin
        if rising_edge(clk) then
            error_packet <= '0';
            if nreset = '0' then
                counter <= (others => '0');
            elsif (current_state = RCV_A) or (current_state = RCV_B) or (current_state = RCV_C) then
                if transmission_valid = '1' then
                    if ((counter+1 > current_len)
                        or ((transmission_valid_last = '0') and (counter+1 = current_len))
                        or ((transmission_valid_last = '1') and (counter+1 < current_len))) then
                        error_packet <= '1';
                    end if;
                    if (transmission_valid_last = '1') then
                        counter <= (others => '0');
                    else
                        counter <= counter+1;
                    end if;
                end if;
            else
                counter <= (others => '0');
            end if;
        end if;
    end process;

    -- signals redirection, based on the current state
    process(clk) begin
        if rising_edge(clk) then
            case current_state is
                when RCV_A =>
                    OUT_A_M_AXIS_tdata <= w_IN_S_AXIS_tdata;
                    OUT_A_M_AXIS_tvalid <= IN_S_AXIS_tvalid;
                    OUT_A_M_AXIS_tlast <= IN_S_AXIS_tlast;

                    OUT_B_M_AXIS_tdata <= (others => '0');
                    OUT_B_M_AXIS_tvalid <= '0';
                    OUT_B_M_AXIS_tlast <= '0';

                    OUT_C_M_AXIS_tdata <= (others => '0');
                    OUT_C_M_AXIS_tvalid <= '0';
                    OUT_C_M_AXIS_tlast <= '0';

                when RCV_B =>
                    OUT_A_M_AXIS_tdata <= (others => '0');
                    OUT_A_M_AXIS_tvalid <= '0';
                    OUT_A_M_AXIS_tlast <= '0';

                    OUT_B_M_AXIS_tdata <= w_IN_S_AXIS_tdata;
                    OUT_B_M_AXIS_tvalid <= IN_S_AXIS_tvalid;
                    OUT_B_M_AXIS_tlast <= IN_S_AXIS_tlast;

                    OUT_C_M_AXIS_tdata <= (others => '0');
                    OUT_C_M_AXIS_tvalid <= '0';
                    OUT_C_M_AXIS_tlast <= '0';

                when RCV_C =>
                    OUT_A_M_AXIS_tdata <= (others => '0');
                    OUT_A_M_AXIS_tvalid <= '0';
                    OUT_A_M_AXIS_tlast <= '0';

                    OUT_B_M_AXIS_tdata <= (others => '0');
                    OUT_B_M_AXIS_tvalid <= '0';
                    OUT_B_M_AXIS_tlast <= '0';

                    OUT_C_M_AXIS_tdata <= w_IN_S_AXIS_tdata;
                    OUT_C_M_AXIS_tvalid <= IN_S_AXIS_tvalid;
                    OUT_C_M_AXIS_tlast <= IN_S_AXIS_tlast;

                when others =>
                    OUT_A_M_AXIS_tdata <= (others => '0');
                    OUT_A_M_AXIS_tvalid <= '0';
                    OUT_A_M_AXIS_tlast <= '0';

                    OUT_B_M_AXIS_tdata <= (others => '0');
                    OUT_B_M_AXIS_tvalid <= '0';
                    OUT_B_M_AXIS_tlast <= '0';

                    OUT_C_M_AXIS_tdata <= (others => '0');
                    OUT_C_M_AXIS_tvalid <= '0';
                    OUT_C_M_AXIS_tlast <= '0';

            end case;
        end if;
    end process;

    Nsm_rtds <= w_nsm_rtds;
    Nsm_rtds_vld <= w_nsm_rtds_vld;

end Behavioral;
